@@ -11,17 +11,18 @@ PYTYPE_TO_CTYPE_TABLE = {
     int: 'long',
     float: 'double',
     str: 'pythonic::types::str',
+    slice: 'pythonic::types::slice',
     type(None): 'pythonic::types::none_type',
-    int8: 'int8_t',
-    int16: 'int16_t',
-    int32: 'int32_t',
-    int64: 'int64_t',
-    intp: 'intptr_t',
-    uint8: 'uint8_t',
-    uint16: 'uint16_t',
-    uint32: 'uint32_t',
-    uint64: 'uint64_t',
-    uintp: 'uintptr_t',
+    int8: 'npy_int8',
+    int16: 'npy_int16',
+    int32: 'npy_int32',
+    int64: 'npy_int64',
+    intp: 'npy_intp',
+    uint8: 'npy_uint8',
+    uint16: 'npy_uint16',
+    uint32: 'npy_uint32',
+    uint64: 'npy_uint64',
+    uintp: 'npy_uintp',
     float32: 'float',
     float64: 'double',
     complex64: 'std::complex<float>',
