@@ -13,7 +13,7 @@ from pythran.passmanager import ModuleAnalysis
 from pythran.tables import operator_to_lambda, MODULES
 from pythran.types.conversion import pytype_to_ctype
 from pythran.types.reorder import Reorder
-from pythran.utils import attr_to_path
+from pythran.utils import attr_to_path, cxxid
 
 from collections import defaultdict
 from functools import partial
@@ -68,18 +68,17 @@ class Types(ModuleAnalysis):
         self.combiners = defaultdict(UserFunction)
         self.current_global_declarations = dict()
         self.max_recompute = 1  # max number of use to be lazy
-        ModuleAnalysis.__init__(self, StrictAliases, LazynessAnalysis,
+        ModuleAnalysis.__init__(self, Reorder, StrictAliases, LazynessAnalysis,
                                 Immediates)
         self.curr_locals_declaration = None
 
-    def prepare(self, node, ctx):
+    def prepare(self, node):
         """
         Initialise values to prepare typing computation.
 
         Reorder functions to avoid dependencies issues and prepare typing
         computation setting typing values for Pythonic functions.
         """
-        self.passmanager.apply(Reorder, node, ctx)
 
         def register(name, module):
             """ Recursively save function typing and combiners for Pythonic."""
@@ -95,10 +94,10 @@ class Types(ModuleAnalysis):
 
         for mname, module in MODULES.items():
             register(mname, module)
-        super(Types, self).prepare(node, ctx)
+        super(Types, self).prepare(node)
 
-    def run(self, node, ctx):
-        super(Types, self).run(node, ctx)
+    def run(self, node):
+        super(Types, self).run(node)
         for head in self.current_global_declarations.values():
             if head not in self.result:
                 self.result[head] = "pythonic::types::none_type"
@@ -171,25 +170,28 @@ class Types(ModuleAnalysis):
         try:
             if register:    # this comes from an assignment,
                             # so we must check where the value is assigned
-                node_id, depth = self.node_to_id(node)
-                if depth > 0:
-                    node = ast.Name(node_id, ast.Load(), None)
-                    former_unary_op = unary_op
+                try:
+                    node_id, depth = self.node_to_id(node)
+                    if depth > 0:
+                        node = ast.Name(node_id, ast.Load(), None)
+                        former_unary_op = unary_op
 
-                    # update the type to reflect container nesting
-                    def unary_op(x):
-                        return reduce(lambda t, n:
-                                      self.builder.ContainerType(t),
-                                      range(depth), former_unary_op(x))
+                        # update the type to reflect container nesting
+                        def unary_op(x):
+                            return reduce(lambda t, n:
+                                          self.builder.ContainerType(t),
+                                          range(depth), former_unary_op(x))
 
-                    # patch the op, as we no longer apply op, but infer content
-                    def op(*types):
-                        if len(types) == 1:
-                            return types[0]
-                        else:
-                            return self.builder.CombinedTypes(*types)
+                        # patch the op, as we no longer apply op, but infer content
+                        def op(*types):
+                            if len(types) == 1:
+                                return types[0]
+                            else:
+                                return self.builder.CombinedTypes(*types)
 
-                self.name_to_nodes[node_id].append(node)
+                    self.name_to_nodes[node_id].append(node)
+                except UnboundableRValue:
+                    pass
 
             # only perform inter procedural combination upon stage 0
             if register and self.isargument(node) and self.stage == 0:
@@ -243,12 +245,15 @@ class Types(ModuleAnalysis):
                 if node not in self.result or self.result[node] is UnknownType:
                     self.result[node] = new_type
                 else:
+                    if isinstance(self.result[node], tuple):
+                        raise UnboundableRValue
                     self.result[node] = op(self.result[node], new_type)
+
         except UnboundableRValue:
             pass
 
     def visit_FunctionDef(self, node):
-        self.curr_locals_declaration = self.passmanager.gather(
+        self.curr_locals_declaration = self.gather(
             LocalNodeDeclarations,
             node)
         self.current = node
@@ -257,7 +262,7 @@ class Types(ModuleAnalysis):
         for arg in node.args.args:
             self.name_to_nodes[arg.id].append(arg)
 
-        self.yield_points = self.passmanager.gather(YieldPoints, node)
+        self.yield_points = self.gather(YieldPoints, node)
 
         # two stages, one for inter procedural propagation
         self.stage = 0
@@ -282,7 +287,7 @@ class Types(ModuleAnalysis):
             self.builder.NamedType("pythonic::types::none_type"))
 
         self.result[node] = self.builder.Returnable(return_type), self.typedefs
-        for k in self.passmanager.gather(LocalNodeDeclarations, node):
+        for k in self.gather(LocalNodeDeclarations, node):
             self.result[k] = self.get_qualifier(k)(self.result[k])
 
     def get_qualifier(self, node):
@@ -474,7 +479,7 @@ class Types(ModuleAnalysis):
             typename = pytype_to_ctype(obj.signature)
             self.result[node] = self.builder.NamedType(typename)
         else:
-            self.result[node] = self.builder.DeclType('::'.join(path) + '{}')
+            self.result[node] = self.builder.DeclType('::'.join(map(cxxid,path)) + '{}')
 
     def visit_Slice(self, node):
         """
